/*
 * GUI.c
 *
 *  Created on: Aug 7, 2013
 *      Author: a0131604
 */


#ifndef GUI_C_
#define GUI_C_
#include "msp430g2553.h"
#include "INSTA_SPIN.h"
#define POLY 				0xA001

#define FRAME_LENGTH		9


unsigned char 				Tx_Buffer [FRAME_LENGTH] = {0,0,0,0,0,0,0,0,0},
							Rx_Buffer [15] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
							tx_ptr = 0,
							rx_ptr = 0,
							Start_Frame = 0,End_Frame = 0,
							UART_Data_Recieved = 0,
							gpctr = 0,
							Rx_Data [20];



unsigned int 				CRC_Initial_Value,
							Int_Constant = 0,
							Init_Ramp_Time = 0,
							Given_Speed = 0;
unsigned char				Prev_Enable_State = 0,
							Prev_Rotation = 0,
							Rotating = 0;




void Process_UART_Data (void);
void Send_UART(void);
unsigned int Calculate_CRC16 (unsigned char *buffer);

void Init_System_Variables(void)
{

INTEGRATION_CONSTANT =  200;
START_UP_DUTY = 1;				//%
START_UP_MAX_DUTY	=  10;		//%
INCREMENTAL_DUTY = 1;			//%
INCREMENTAL_TIME = 4;			//mS
WAIT_TIME =  10;				//mS
SPEED_DUTY_MAX = 97;			//%
SPEED_DUTY_MIN = 10;			//%
SPEED_RAMP_CYCLES = 2;
Hysteritic_Control = 2;
Speed_Control = 0;
Rotation = 0;

}

void mode_initialize()
{
#ifdef GUI

	State = 0 ;
#endif

#ifndef GUI
	State = 1 ;
#endif
}

void Process_UART_Data (void)
{
	unsigned int CRC_Value = 0;

	if (UART_Data_Recieved)
	{
		UART_Data_Recieved = 0;
			switch (Rx_Buffer [1])
			{
				case 'A':
					Send_UART ();
					break;
				case 'B':
					if (Start_Braking != 1)
					{
						Speed_Control = Rx_Buffer [8];
						switch (Speed_Control)
						{
							case 1 :
								Speed_Duty_Cycle = (100 - Rx_Buffer [11]) << 2;
								break;
							case 2 :
								Given_Speed = (Rx_Buffer [10] << 8) | Rx_Buffer [11];
								Required_Speed = Given_Speed;
								break;
							default:
								break;
						}

						Rotation = Rx_Buffer [9];
						if ((Rotation != Prev_Rotation) && (Rotating))
						{
							Rotation = Prev_Rotation;
							Start_Braking = 1;
							Speed_Control = 0;
						}

						Int_Constant = (Rx_Buffer [3] << 8) | (Rx_Buffer [4]);
						INTEGRATION_CONSTANT = Int_Constant;
						START_UP_MAX_DUTY = Rx_Buffer [5];
						Start_Max_Duty_Cycle = (100 - START_UP_MAX_DUTY) * 4;
						Init_Ramp_Time = (Rx_Buffer [6] << 8) | (Rx_Buffer [7]);
						WAIT_TIME = Init_Ramp_Time >> 1 ;


						if (Rx_Buffer [2] != Prev_Enable_State)
						{
							if (Rx_Buffer [2] == 0x01)
							{
								State = 1;
								Prev_Rotation = Rotation ;
								Rotating = 1;
							}
							else
							{
								High_Impedance ();
								_disable_interrupt ();
								P3OUT &= ~(BIT4+BIT5);
								_delay_cycles (16000000);
								WDTCTL = 0xDEAD;
								while (1);
							}
							Prev_Enable_State = Rx_Buffer [2];
						}


						if (Rx_Buffer [12] == 0x01)
						{
							High_Impedance ();
							_disable_interrupt ();
							P3OUT &= ~(BIT4+BIT5);
							_delay_cycles (16000000);
							WDTCTL = 0xDEAD;
							while (1);
						}
					}
					Send_UART ();
					break;
				default:
					break;
			}


	}
	else
	{
		Int_Constant = 0;
		Init_Ramp_Time = 0;
		Given_Speed = 0;
	}
}



void Send_UART(void)
{
	unsigned int CRC_Value = 0;

	Tx_Buffer [0] = '*';
	Tx_Buffer [1] = 0x31;
	Tx_Buffer [2] = Rotation;
	Tx_Buffer [3] = (Actual_Speed >> 8) & 0xFF;
	Tx_Buffer [4] = Actual_Speed & 0xFF;
	Tx_Buffer [5] = (DC_Bus_Voltage >> 8) & 0xFF;
	Tx_Buffer [6] = DC_Bus_Voltage & 0xFF;

	tx_ptr = 0;
	IE2 |= UCA0TXIE;
}



#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
{
	Rx_Data [rx_ptr] = UCA0RXBUF;



	if ((Rx_Data [rx_ptr] == '#')&&(Rx_Data [rx_ptr - 14] == '*' ))
	{
		End_Frame = rx_ptr;
		for (gpctr = 0 ; gpctr < 15 ; gpctr++ )
		{
			Rx_Buffer [gpctr] = Rx_Data [End_Frame - 14 + gpctr ];
		}
		rx_ptr = 0;
		UART_Data_Recieved= 1;
	}
	else
		rx_ptr++;

	if (rx_ptr > 19) rx_ptr = 0;
}



#pragma vector=USCIAB0TX_VECTOR
__interrupt void USCI0TX_ISR(void)
{
	UCA0TXBUF = Tx_Buffer [tx_ptr];                 // TX next character
	tx_ptr++;
	if (tx_ptr == FRAME_LENGTH)
	{
		tx_ptr = 0;
		IE2 &= ~UCA0TXIE;
	}
}


#endif /* GUI_C_ */




