/*
 * Main.c
 *
 *  Created on: Aug 7, 2013
 *      Author: a0131604
 */


#include "msp430g2553.h"
#include "INSTA_SPIN.h"

void setupUART115200();
void Init_Clocks ();
void Init_Timer ();
void Init_IOs (void);
void Init_ADC ();
void All_Reset_ON(void);
void Process_UART_Data ();
void Init_System_Variables ();
void mode_initialize();

#define CONVERSION_ENABLE        \
		ADC10CTL0 |= ENC;





void main()
{

	WDTCTL = WDTPW + WDTHOLD;				// Stop watch-dog timer to prevent time out reset
	_delay_cycles (5000);
	Init_Clocks ();
	Init_IOs ();
	Init_Timer ();

#ifdef GUI
	setupUART115200();
#endif

	Init_ADC ();
	Init_System_Variables ();
	mode_initialize();
	CONVERSION_ENABLE;
 	__bis_SR_register(GIE);                   // LPM0, ADC10_ISR will force exit


 	Init_InstaSpin();


	while(1)
	{
		InstaSpin();

#ifdef GUI
		Process_UART_Data ();
#endif

	}

}



void Init_ADC ()
{
	ADC10CTL0 |= SREF_1 + ADC10SHT_2 + ADC10ON + ADC10IE + REFON + REF2_5V;
	ADC10CTL1 = INCH_6 + CONSEQ_0 + ADC10SSEL_2 ;			// Sequence of channels upto A6
	ADC10AE0  |= 0x40;
}



void Init_IOs (void)
{
	// setup for PWMC
	P3DIR |= 0x3F;				//P3.0 to P3.5 as Outputs
	P3SEL |= BIT3;				//P3.3 as PWM

	P2DIR |= (BIT4 + BIT5);		// P2.4 and 5 as PWM Outputs
	P2SEL |= (BIT4 + BIT5);		// Bit Setting for PWM Outputs
	//P2DIR |= BIT7;
	P3OUT &= ~(BIT4+BIT5);
	P3DIR |= BIT7;
}



void Init_Timer (void)
{
	TA1CCR0		= 400;							// Load the Reference Count
	TA1CCTL2	= OUTMOD_6;
	TA1CCR2		= (100-START_UP_DUTY) * 4;
	TA1CTL		= TASSEL_2 + MC_3 + TACLR ;		//+ TAIE;	//Up-Down Mode , SMCLK , Timer Clear

	TACCR0		= 400;							// Load the Reference Count
	TACCTL0		= CCIE;
	TACCTL2		= OUTMOD_6;
	TACCR2		= 398;
	TACTL		= TASSEL_2 + MC_3 + TACLR ;		//+ TAIE;	//Up-Down Mode , SMCLK , Timer Clear


	WDTCTL 		= WDT_MDLY_32;
	IE1 		|= WDTIE;                         // enable WDT interrupt
	_EINT();                              // Enable interrupts
}



void setupUART115200(void)
{
	//Setup USCI to UART on P1.1 and P1.2
	P1SEL = BIT1 + BIT2 ;                     // P1.1 = RXD, P1.2=TXD
	P1SEL2 = BIT1 + BIT2 ;                     // P1.1 = RXD, P1.2=TXD
	UCA0CTL1 |= UCSSEL_2;                     // SMCLK
	UCA0BR0 =138;                            // 16MHz 115200
	UCA0BR1 = 0;                              // 16MHz 115200
	UCA0MCTL = UCBRS_7;                        // Modulation UCBRSx = 1
	UCA0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**
	IE2 |= UCA0RXIE;                          // Enable USCI_A0 RX interrupt
}



void Init_Clocks (void)
{
	BCSCTL1		= CALBC1_16MHZ;               	// Set DCO ~ 16 Mhz
	DCOCTL		= CALDCO_16MHZ;
	BCSCTL1 	|= DIVA_1;                    	// ACLK/(0:1,1:2,2:4,3:8)
	BCSCTL3 	|= LFXT1S_2;                  	// LFXT1 = VLO
}



void All_Reset_ON(void)
{
	P3OUT |= (BIT0+BIT1+BIT2);
}



#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A0 (void)
{
	Measure_Speed++;

}
